/*
 * Copyright (c) 2014, Oracle and/or its affiliates. All rights reserved.
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
 *
 * This code is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 only, as
 * published by the Free Software Foundation.
 *
 * This code is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * version 2 for more details (a copy is included in the LICENSE file that
 * accompanied this code).
 *
 * You should have received a copy of the GNU General Public License version
 * 2 along with this work; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * Please contact Oracle, 500 Oracle Parkway, Redwood Shores, CA 94065 USA
 * or visit www.oracle.com if you need additional information or have any
 * questions.
 */

/**
 * @test TestDynShrinkHeap
 * @bug 8037924
 * @summary Verify that heap shrinking after full gc and dynamically changed free ratio
 * @author Andrey Zakharov /andrey.x.zakharov@oracle.com/
 * @library /testlibrary
 * @run main/othervm -XX:+UseAdaptiveSizePolicyWithSystemGC -XX:+UseParallelGC -XX:MinHeapFreeRatio=0 -XX:MaxHeapFreeRatio=100 -Xloggc:gc.log TestDynShrinkHeap
 */

import com.oracle.java.testlibrary.TestDynamicVMOption;
import java.util.ArrayList;

import com.oracle.java.testlibrary.MemoryUsagePrinter;
import com.oracle.java.testlibrary.TestShrinkHeap;

public class TestDynShrinkHeap extends TestShrinkHeap {

    public static ArrayList<byte[]> list = new ArrayList<>(0);
    public static final int M = 1024 * 1024; // to make heap more manageable by test code
     
    protected final TestDynamicVMOption minRatioOption;
    protected final TestDynamicVMOption maxRatioOption;   
    
    private int minHeapFreeValue;
    private int maxHeapFreeValue;

    public TestDynShrinkHeap() {
        super();
        minRatioOption = new TestDynamicVMOption(MIN_FREE_RATIO_FLAG_NAME);
        maxRatioOption = new TestDynamicVMOption(MAX_FREE_RATIO_FLAG_NAME);

        minHeapFreeValue = minRatioOption.getIntValue();
        maxHeapFreeValue = maxRatioOption.getIntValue();
        // with maxHeapFreeRatio = 100 heap will not shrink at all, so
        if (maxHeapFreeValue == 100) {
            maxHeapFreeValue = 95;
            maxRatioOption.setIntValue(maxHeapFreeValue);
        }

    }
    
    public void eat() {
        for (int i = 0; i < M; i++) {
            list.add(new byte[1024]);
        }

        MemoryUsagePrinter.printMemoryUsage("allocated " + M + " arrays");
    }

    public void free() {

        list.subList(0, M / 2).clear();
        gc();
        MemoryUsagePrinter.printMemoryUsage("array halfed");
        
        maxRatioOption.setIntValue(minHeapFreeValue + 1);
        gc();
        MemoryUsagePrinter.printMemoryUsage("under pressure");
    }

    public static void main(String[] args) {
        new TestDynShrinkHeap().test();
    }

    public static void gc() {
        MemoryUsagePrinter.printMemoryUsage("before full GC");
        System.gc();
        MemoryUsagePrinter.printMemoryUsage("after full GC");
    }

}
